cda = {}

require('cda.codesystem')
require('cda.util')
require('cda.header')
require('cda.header.patient')
require('cda.header.serviceevent')
require('cda.participant')
require('cda.document')
require('cda.section')
require('cda.entry')
require('cda.observation') -- important that this is after cda.entry
require('cda.substanceadmin') -- important that this is after cda.entry
require('cda.reference')

require('dateparse')
require('xml')


local CDA_DOC = [[<?xml version="1.0"?>
<ClinicalDocument xmlns="urn:hl7-org:v3" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
	<realmCode code="US"/>
	<typeId root="2.16.840.1.113883.1.3" extension="POCD_HD000040"/>
	<id extension="#" root="#"/>
	<code code="#" codeSystem="2.16.840.1.113883.6.1" codeSystemName="LOINC" displayName="#"/>
	<title>#</title>
	<effectiveTime value="#"/>
	<confidentialityCode code="#" codeSystem="2.16.840.1.113883.5.25" codeSystemName="Confidentiality" displayName="#"/>
	<languageCode code="en-US"/>
</ClinicalDocument>
]]

function debug(N) return end

-- Header for the generated XML document.
cda.XML_HEADER = [[<?xml version="1.0"?>
]]

-- If set, will be in the xsi:schemaLocation attribute of the document.
cda.SCHEMA_LOCATION = 'CDA.xsd'

function cda.new()
   local Doc = xml.parse{data=CDA_DOC}
   if cda.SCHEMA_LOCATION then
      cda.setAttr(Doc, 'xsi', 'schemaLocation', 'urn:hl7-org:v3 '..cda.SCHEMA_LOCATION)
   end
   return Doc
end

function cda.setAttr(Doc, Namespace, Name, Value)
   Doc.ClinicalDocument:setAttr(Namespace..':'..Name, Value)
end

function cda.createBody(Doc)
   local C = imp.getOrCreateElement(Doc.ClinicalDocument, 'component')
   return imp.getOrCreateElement(C, 'structuredBody')
end

function cda.serialize(Doc)
   return cda.XML_HEADER..tostring(Doc)
end
