cda.codesystem = {}

------------------------------------------------------------------------
-- We provide a few codesystems here which can be extended as needed. --
-- Alternatively you can define other id/name pairs to use anywhere   --
-- else you like.                                                     --
------------------------------------------------------------------------

cda.codesystem.codeSystems = {
   NCI_FDA_RouteOfAdministration = {id='2.16.840.1.113883.3.26.1.1', name='NCI - FDA RouteOfAdministration'},
   NCI_FDA_DosageForms           = {id='2.16.840.1.113883.3.26.1.1', name='NCI - FDA Dosage Forms'},
   Gender                        = {id='2.16.840.1.113883.5.1',      name='HL7 AdministrativeGender'},
   MaritalStatus                 = {id='2.16.840.1.113883.5.2',      name='HL7 Marital status'},
   IntoleranceType               = {id='2.16.840.1.113883.5.4',      name='ObservationIntoleranceType'},
   ActCode                       = {id='2.16.840.1.113883.5.4',      name='HL7 ActCode'},
   ActStatus                     = {id='2.16.840.1.113883.5.14',     name='ActStatus'},
   Confidentiality               = {id='2.16.840.1.113883.5.25',     name='Confidentiality'},
   LanguageAbility               = {id='2.16.840.1.113883.5.60',     name='LanguageAbilityMode'},
   ObservationInterpretation     = {id='2.16.840.1.113883.5.83',     name='ObservationInterpretation'},
   RoleCode                      = {id='2.16.840.1.113883.5.111',    name='RoleCode'},
   RoleCodePersonal              = {id='2.16.840.1.113883.5.111',    name='HL7 RoleCode (Personal Relationship subset)'},
   RouteOfAdministration         = {id='2.16.840.1.113883.5.112',    name='HL7 RouteOfAdministration'},
   ServiceDeliveryLocation       = {id='2.16.840.1.113883.5.10588',  name='HL7 Service Delivery Location'},
   LOINC                         = {id='2.16.840.1.113883.6.1',      name='LOINC'},
   ICD9CM                        = {id='2.16.840.1.113883.6.2',      name='ICD9CM'},
   UCUM                          = {id='2.16.840.1.113883.6.8',      name='UCUM'},
   RxNorm                        = {id='2.16.840.1.113883.6.88',     name='RxNorm'},
   SNOMED_CT                     = {id='2.16.840.1.113883.6.96',     name='SNOMED CT'},
   SNOMED                        = {id='2.16.840.1.113883.6.96',     name='SNOMED'},
   ProviderCodes                 = {id='2.16.840.1.113883.6.101',    name='Provider Codes'},
   ICD9CM_Diagnosis              = {id='2.16.840.1.113883.6.103',    name='ICD-9CM (diagnoses)'},
   ICD9CM_Procedure              = {id='2.16.840.1.113883.6.104',    name='ICD-9CM (procedure codes)'},
   RaceAndEthnicity              = {id='2.16.840.1.113883.6.238',    name='HL7 Race and Ethnicity'},
   ProviderRole                  = {id='2.16.840.1.113883.12.443',   name='HL7 Provider Role'}
}

cda.cs = cda.codesystem.codeSystems -- shorthand

local CODESYSTEM_SET_USAGE =
[[Sets the codeSystem and codeSystemName attributes for an element.
Accepts two parameters:
 1) The element whose attributes should be set, and
 2) The codesystem to use (a table from cda.cs, or a custom-defined table).
    Should have two entries, 'id' and 'name'.]]

function cda.codesystem.set(E, CodeSystem)
   if type(CodeSystem) ~= 'table' then
      error(CODESYSTEM_SET_USAGE, 2)
   end
   E:setAttr('codeSystem', CodeSystem.id)
   E:setAttr('codeSystemName', CodeSystem.name)
end
