cda.document = {}

local findFirstElementIndex
local findTemplateIdInsertionIndex


-------------------------------------------------------------------
-- Functions in cda.document accept various 'document' elements, --
-- such as <ClinicalDocument> and <parentDocument>.              --
-------------------------------------------------------------------

-- AssigningAuthorityName is optional
function cda.document.addTemplateId(D, Root, AssigningAuthorityName)
   local InsertionIndex = findTemplateIdInsertionIndex(D)
   local TI = D:insert(InsertionIndex, xml.ELEMENT, 'templateId')
   TI:setAttr('root', Root)
   if AssigningAuthorityName then
      TI:setAttr('assigningAuthorityName', AssigningAuthorityName)
   end
   return TI
end

-- Extension and AssigningAuthorityName are optional
function cda.document.setId(D, Root, Extension, AssigningAuthorityName)
   local I = imp.getOrCreateElement(D, 'id')
   I:setAttr('root', Root)
   if Extension then
      I:setAttr('extension', Extension)
   end
   if AssigningAuthorityName then
      I:setAttr('assigningAuthorityName', AssigningAuthorityName)
   end
   return I
end

function cda.document.setCode(D, Code, DisplayName)
   local C = imp.getOrCreateElement(D, 'code')
   C:setAttr('code', Code)
   C:setAttr('displayName', DisplayName)
   return C
end

function cda.document.setTitle(D, Title)
   local T = imp.getOrCreateElement(D, 'title')
   T:setText(Title)
   return T
end

-- Default precision is 3 (YYYYMMDD)
function cda.document.setEffectiveTime(D, FuzzyDate, Precision)
   return cda.util.date.setDate(D.effectiveTime, FuzzyDate, Precision)
end

function cda.document.setConfidentialityCode(D, Code, DisplayName)
   local CC = imp.getOrCreateElement(D, 'confidentialityCode')
   CC:setAttr('code', Code)
   if DisplayName then
      CC:setAttr('displayName', DisplayName)
   end
   return CC
end

-- Extension is optional
function cda.document.setSetId(D, Root, Extension)
   local SI = D:append(xml.ELEMENT, 'setId')
   SI:setAttr('root', Root)
   if Extension then
      SI:setAttr('extension', Extension)
   end
   return SI
end

function cda.document.setVersionNumber(D, VersionNumber)
   local VN = imp.getOrCreateElement(D, 'versionNumber')
   VN:setAttr('value', VersionNumber)
   return VN
end

-- Will set the attribute 'mediaType' to MediaType,
-- if specified.
function cda.document.addText(D, MediaType)
   local T = D:append(xml.ELEMENT, 'text')
   if MediaType then
      T:setAttr('mediaType', MediaType)
   end
   return T
end


-- Local functions

function findTemplateIdInsertionIndex(D)
   -- templateId elements should appear before id
   for i=1,#D do
      if D[i]:nodeType() == 'element' and D[i]:nodeName() == 'id' then
         return i
      end
   end
   return #D
end
