cda.header.patient = {}
cda.header.patient.guardian = {}

-------------------------------------------------------------------------
-- All functions in cda.header.patient accept a <patientRole> element. --
-------------------------------------------------------------------------

-- Use is optional
function cda.header.patient.addName(PR, Arg, Use)
   local P = imp.getOrCreateElement(PR, 'patient')
   return cda.util.addName(P, Arg, Use)
end

-- DisplayName is optional
function cda.header.patient.setGender(PR, G, DisplayName)
   local P = imp.getOrCreateElement(PR, 'patient')
   local AGC = imp.getOrCreateElement(P, 'administrativeGenderCode')
   AGC:setAttr('code', G)
   if DisplayName then
      AGC:setAttr('displayName', DisplayName)
   end
   cda.codesystem.set(AGC, cda.cs.Gender)
   return AGC
end

function cda.header.patient.setBirthtime(PR, FuzzyDate)
   local P = imp.getOrCreateElement(PR, 'patient')
   local BT = imp.getOrCreateElement(P, 'birthTime')
   return cda.util.date.setDate(BT, FuzzyDate)
end

function cda.header.patient.setMaritalStatus(PR, Code, DisplayName)
   local P = imp.getOrCreateElement(PR, 'patient')
   local MSC = imp.getOrCreateElement(P, 'maritalStatusCode')
   MSC:setAttr('code', Code)
   if DisplayName then
      MSC:setAttr('displayName', DisplayName)
   end
   cda.codesystem.set(MSC, cda.cs.MaritalStatus)
   return MSC
end

function cda.header.patient.setRace(PR, Code, DisplayName)
   local P = imp.getOrCreateElement(PR, 'patient')
   local RC = imp.getOrCreateElement(P, 'raceCode')
   RC:setAttr('code', Code)
   if DisplayName then
      RC:setAttr('displayName', DisplayName)
   end
   cda.codesystem.set(RC, cda.cs.RaceAndEthnicity)
   return RC
end

function cda.header.patient.setEthnicGroup(PR, Code, DisplayName)
   local P = imp.getOrCreateElement(PR, 'patient')
   local EGC = imp.getOrCreateElement(P, 'ethnicGroupCode')
   EGC:setAttr('code', Code)
   if DisplayName then
      EGC:setAttr('displayName', DisplayName)
   end
   cda.codesystem.set(EGC, cda.cs.RaceAndEthnicity)
   return EGC
end

-- Use cda.header.patient.guardian functions with return value
function cda.header.patient.addGuardian(PR, Code, DisplayName)
   local P = imp.getOrCreateElement(PR, 'patient')
   local G = P:append(xml.ELEMENT, 'guardian')
   G:setAttr('classCode', 'GUARD')
   G:append(xml.ELEMENT, 'templateId'):setAttr('root', '2.16.840.1.113883.3.88.11.83.3')
   G:append(xml.ELEMENT, 'templateId'):setAttr('root', '1.3.6.1.4.1.19376.1.5.3.1.2.4')
   local C = G:append(xml.ELEMENT, 'code')
   C:setAttr('code', Code)
   if DisplayName then
      C:setAttr('displayName', DisplayName)
   end
   cda.codesystem.set(C, cda.cs.RoleCode)
   return G
end

local LANGUAGE_COMUNICATION_TEMPLATE_IDS = [[
<templateId root="2.16.840.1.113883.3.88.11.83.2" assigningAuthorityName="HITSP/C83"/>
<templateId root="1.3.6.1.4.1.19376.1.5.3.1.2.1" assigningAuthorityName="IHE/PCC"/>
]]

-- ModeCode and ModeDisplayName are optional
function cda.header.patient.addLanguageCommunication(PR, LanguageCode, Preference, ModeCode, ModeDisplayName)
   local P = imp.getOrCreateElement(PR, 'patient')
   local LC = P:append(xml.ELEMENT, 'languageCommunication')
   LC:setInner(LANGUAGE_COMUNICATION_TEMPLATE_IDS)
   LC:append(xml.ELEMENT, 'languageCode'):setAttr('code', LanguageCode)
   if ModeCode or ModeDisplayName then
      local MC = LC:append(xml.ELEMENT, 'modeCode')
      MC:setAttr('code', ModeCode)
      MC:setAttr('displayName', ModeDisplayName)
      cda.codesystem.set(MC, cda.cs.LanguageAbility)
   end
   LC:append(xml.ELEMENT, 'preferenceInd'):setAttr('value', tostring(Preference))
   return LC
end

-- Extension is optional
function cda.header.patient.setProviderOrganization(PR, Root, Extension)
   return imp.getOrCreateChildWithId(PR, 'providerOrganization', Root, Extension)
end


-------------------------------------------------------------------------------
-- All functions in cda.header.patient.guardian accept a <guardian> element. --
-------------------------------------------------------------------------------

function cda.header.patient.guardian.addName(G, Arg, Use)
   local GP = imp.getOrCreateElement(G, 'guardianPerson')
   return cda.util.addName(GP, Arg, Use)
end
