cda.observation = {}

-- Copy everything from cda.entry into cda.observation.
-- Conceptually, observation "inherits" from entry.
for K,V in pairs(cda.entry) do
   cda.observation[K] = V
end

-------------------------------------------------------------------
-- Functions in cda.observation accept an <observation> element. --
-------------------------------------------------------------------

-- DisplayName is optional
function cda.observation.addMethodCode(O, Code, CodeSystem, DisplayName)
   return cda.util.addCodeElement(O, 'methodCode', Code, CodeSystem, DisplayName)
end

-- DisplayName is optional
function cda.observation.addInterpretationCode(O, Code, DisplayName)
   cda.util.addCodeElement(O, 'interpretationCode', Code, cda.cs.ObservationInterpretation, DisplayName)
end

-- Numerator and Denominator should be tables with keys
-- 'value' and 'unit'.
function cda.observation.addRtoPqPqValue(O, Numerator, Denominator)
   local V = O:append(xml.ELEMENT, 'value')
   V:setAttr('xsi:type', 'RTO_PQ_PQ')
   
   local N = V:append(xml.ELEMENT, 'numerator')
   N:setAttr('value', Numerator.value)
   N:setAttr('unit',  Numerator.unit)
   
   local D = V:append(xml.ELEMENT, 'denominator')
   D:setAttr('value', Denominator.value)
   D:setAttr('unit',  Denominator.unit)
   
   return V
end

-- XsiType is optional
function cda.observation.addValueWithCode(O, Code, DisplayName, CodeSystem, XsiType)
   local V = cda.util.addCodeElement(O, 'value', Code, DisplayName, CodeSystem)
   cda.util.setXsiType(V, XsiType)
   return V
end

function cda.observation.addReference(O, TypeCode)
   local R = O:append(xml.ELEMENT, 'reference')
   R:setAttr('typeCode', TypeCode)
   return R
end

local RELATED_SUBJECT_TEMPLATE = [[
<relatedSubject classCode="#">
   <code code="#"/>
</relatedSubject>
]]

function cda.observation.addRelatedSubject(O, ClassCode, Code)
   local S = O:append(xml.ELEMENT, 'subject')
   S:setInner(RELATED_SUBJECT_TEMPLATE)
   local RS = S.relatedSubject
   RS.classCode = ClassCode
   RS.code.code = Code
   cda.codesystem.set(RS.code, cda.cs.RoleCode)
   return RS
end

function cda.observation.addParticipantRole(O, TypeCode, RoleClassCode)
   local P = O:append(xml.ELEMENT, 'participant')
   P:setAttr('typeCode', TypeCode)
   local PR = P:append(xml.ELEMENT, 'participantRole')
   PR:setAttr('classCode', RoleClassCode)
   PR:append(xml.ELEMENT, 'addr') -- empty
   PR:append(xml.ELEMENT, 'telecom') -- empty
   return PR
end
