cda.substanceadmin = {}

cda.substanceadmin.manufactured = {}

-- Copy everything from cda.entry into cda.substanceadmin.
-- Conceptually, substanceadmin "inherits" from entry.
for K,V in pairs(cda.entry) do
   cda.substanceadmin[K] = V
end

----------------------------------------------------------------------------------
-- Functions in cda.substanceadmin accept an <substanceAdministration> element. --
-- Other functions that operation on <substanceAdministration> elements can  be --
-- found in cda.entry.                                                          --
----------------------------------------------------------------------------------

function cda.substanceadmin.addEffectiveTime(SA, InstitutionSpecified, Value, Unit)
   local ET = SA:append(xml.ELEMENT, 'effectiveTime')
   ET:setAttr('xsi:type', 'PIVL_TS')
   ET:setAttr('institutionSpecified', tostring(InstitutionSpecified))
   local P = imp.getOrCreateElement(ET, 'period')
   if Value then
      P:setAttr('value', Value)
   end
   if Unit then
      P:setAttr('unit', Unit)
   end
   return ET
end

function cda.substanceadmin.setPriorityCode(SA, Code)
   local PC = imp.getOrCreateElement(SA, 'priorityCode')
   PC:setAttr('code', Code)
   return PC
end

-- DisplayName is optional
function cda.substanceadmin.setRouteCode(SA, Code, CodeSystem, DisplayName)
   return cda.util.addCodeElement(
      SA, 'routeCode', Code, CodeSystem, DisplayName)
end

-- Unit is optional
function cda.substanceadmin.setDoseQuantity(SA, Value, Unit)
   local DQ = imp.getOrCreateElement(SA, 'doseQuantity')
   DQ:setAttr('value', Value)
   if Unit then
      DQ:setAttr('unit', Unit)
   end
   return DQ
end

-- Use cda.substanceadmin.manufactured with the return value
function cda.substanceadmin.addManufacturedLabeledDrug(SA, ProductClassCode)
   local C = SA:append(xml.ELEMENT, 'consumable')
   local MP = C:append(xml.ELEMENT, 'manufacturedProduct')
   if ProductClassCode then
      MP:setAttr('classCode', ProductClassCode)
   end
   return MP:append(xml.ELEMENT, 'manufacturedLabeledDrug')
end

-- Use cda.substanceadmin.manufactured with the return value
function cda.substanceadmin.addManufacturedMaterial
(
   SA, ProductClassCode, MaterialClassCode, DeterminerCode, TemplateIds
)
   local C = SA:append(xml.ELEMENT, 'consumable')
   local MP = C:append(xml.ELEMENT, 'manufacturedProduct')
   if ProductClassCode then
      MP:setAttr('classCode', ProductClassCode)
   end
   if TemplateIds then
      cda.util.addTemplateIds(MP, TemplateIds)
   end
   local MM = MP:append(xml.ELEMENT, 'manufacturedMaterial')
   if MaterialClassCode then
      MM:setAttr('classCode', MaterialClassCode)
   end
   if DeterminerCode then
      MM:setAttr('determinerCode', DeterminerCode)
   end
   return MM
end

function cda.substanceadmin.setApproachSiteCode(SA, Code, CodeSystem, DisplayName)
   return cda.util.addCodeElement(SA, 'approachSiteCode', Code, CodeSystem, DisplayName)
end


-------------------------------------------------------------------
-- Functions in cda.substanceadmin.manufactured accept           --
-- <manufacturedLabeledDrug> or <manufacturedMaterial> elements. --
-------------------------------------------------------------------

-- Use functions in cda.substanceadmin.code with the return value
function cda.substanceadmin.manufactured.addCode(M, Code, CodeSystem, DisplayName)
   return cda.util.addCodeElement(M, 'code', Code, CodeSystem, DisplayName)
end

function cda.substanceadmin.manufactured.setName(M, Name)
   local N = imp.getOrCreateElement(M, 'name')
   N:setText(Name)
   return N
end
